"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const assert = require("assert");
const uuid = require("uuid");
const queue_1 = require("./queue");
const log4js = require("@log4js-node/log4js-api");
const stream_1 = require("stream");
const encode_decode_1 = require("./encode-decode");
const logger = log4js.getLogger('rabbit-queue');
let replyHandlers = {};
let streamHandlers = {};
function createReplyQueue(channel) {
    return __awaiter(this, void 0, void 0, function* () {
        yield channel.assertQueue('', { exclusive: true }).then(replyTo => {
            channel.replyName = replyTo.queue;
            channel.consume(channel.replyName, onReply, { noAck: true });
        });
    });
}
exports.createReplyQueue = createReplyQueue;
function addHandler(correlationId, handler) {
    assert(!replyHandlers[correlationId], `Already added reply handler with this id: ${correlationId}.`);
    assert(!streamHandlers[correlationId], `Already exists stream handler with this id: ${correlationId}.`);
    replyHandlers[correlationId] = handler;
}
exports.addHandler = addHandler;
function getReply(content, properties = {}, channel, cb) {
    return new Promise((resolve, reject) => {
        var correlationId = properties.correlationId || uuid.v4();
        properties = Object.assign({
            persistent: false,
            correlationId,
            replyTo: channel.replyName,
            contentType: 'application/json'
        }, properties);
        const bufferContent = encode_decode_1.encode(content, properties.contentType);
        addHandler(correlationId, (err, body) => (err ? reject(err) : resolve(body)));
        cb(bufferContent, properties, correlationId, (err, ok) => (err ? reject(err) : {}));
    });
}
exports.getReply = getReply;
function onReply(msg) {
    const id = msg.properties.correlationId;
    const headers = msg.properties.headers || {};
    if (headers.isStream) {
        return handleStreamReply(msg, id);
    }
    const replyHandler = replyHandlers[id];
    if (!replyHandler) {
        logger.error(`No reply Handler found for ${id}`);
        return;
    }
    delete replyHandlers[id];
    logger.info(`[${id}] <- Returning reply ${msg.content.byteLength} bytes`);
    const obj = encode_decode_1.decode(msg);
    if (obj && obj.error && obj.error_code === queue_1.default.ERROR_DURING_REPLY.error_code) {
        replyHandler(new Error(obj.error_message), null);
    }
    else {
        replyHandler(null, obj);
    }
}
function handleStreamReply(msg, id) {
    const replyHandler = replyHandlers[id];
    let streamHandler = streamHandlers[id];
    if (replyHandler && streamHandler) {
        delete replyHandlers[id];
        return replyHandler(new Error(`Both replyHandler and StreamHandler exist for id: ${id}`));
    }
    if (!streamHandler) {
        if (!replyHandler) {
            logger.error(`No reply Handler found for ${id}`);
            return;
        }
        delete replyHandlers[id];
        streamHandler = streamHandlers[id] = new stream_1.Readable({ objectMode: true, read() { } });
        replyHandler(null, streamHandler);
    }
    const obj = encode_decode_1.decode(msg);
    logger.info(`[${id}] <- Returning${obj === null && ' the end of'} stream reply ${msg.content.byteLength} bytes`);
    streamHandler.push(obj);
    if (obj === null) {
        delete streamHandlers[id];
    }
}
//# sourceMappingURL=reply-queue.js.map